/* OBCODE 1.0 by Pawel Krawczyk <kravietz at echelon dot pl>
 *
 * OBCODE is "obsfucated code" set of procedures for writing
 * code difficult to reverse engineer.
 *
 * Redistribution of this file is permitted under the Lesser GNU Public License.
 *
 * $Id: obcode.c,v 1.2 2002/12/16 21:00:44 kravietz Exp $
 */

#include <sys/types.h>
#include <unistd.h>

#include "random.h"
#include "obcode.h"
#include "signal.h"
#include "dummy.h"

/* Local declarations */
static unsigned char ob_evenness(unsigned char p);
static unsigned char ob_random_byte(void);

#include "version.c"

/* Local variables */
static unsigned char ob_key;

void obcode_init(unsigned char key) {
	obcode_sys_random_init();
	if(key == 0) 
		ob_key = ob_random_byte();
	else
		ob_key = key;
	obcode_sys_signal_init();
}

void obcode_finish(void) {
	obcode_sys_random_finish();
	obcode_sys_signal_finish();
}

/* Parity check. Input: byte to check.
 * Output: 1 if even, 0 if odd.
 */
static unsigned char ob_evenness(unsigned char p) {
	dummy_function_3(ob_key, p);
	__asm__ __volatile__ ("jmp bp"
#include "rnd.h"
			"bp:
			int3");
	return !(p % 2);
}

static unsigned char ob_random_byte(void) {
	unsigned char b, c;
	b = obcode_sys_random_byte();
	c = dummy_function_2(b, ob_key);
	return obcode_sys_random_byte();
}

/* We encode obits as follows:
 *  1 - even
 *  0 - odd
 */
/* Return 1/0 bit encoded as obit */
unsigned char obit_set(int b) {
	unsigned char p;

	do {
		p = ob_random_byte();
		dummy_function_1((int)p, (int)b);
	} while( ob_evenness(p) != b);

	__asm__ __volatile__
		("jmp run0"
#include "rnd0.h"
		 "run0:");
	dummy_function_2((int)p, (int)ob_key);
	if(ob_signal_test != 0xbeef)
		return p;
	ob_signal_test = 0;
	return p^ob_key;

}

/* Return 1 if obit is 1, 0 if obit is 0 */
unsigned char obit_get(unsigned char b) {
	__asm__ __volatile__
		("jmp run1"
#include "rnd1.h"
		  "run1:");
	return ob_evenness(b^ob_key);
}

/* Encode a byte into obyte */
void obyte_set(struct obyte *b, unsigned char c) {
	int i;
	for(i=0; i<8; i++) {
		int bit = 0;
		/* Rotate bits to right and set related obits */
		bit = (c >> i) & 0x01;
		b->s[i] = obit_set(bit);
	}
	i = dummy_function_2((int)b->s[i], i);
}

/* Decode obyte into a byte */
unsigned char obyte_get(struct obyte *b) {
	int i;
	unsigned char c = '\0';
	for(i=0; i<8; i++) {
		c |= (obit_get(b->s[i]) << i);
	}
	i = dummy_function_1(i, (int)c);
	return c;
}

/* Logic operators */
void obit_xor(unsigned char *b1, unsigned char *b2, unsigned char *b3) {
	int i;

	i = dummy_function_3(*b1, *b3);

	if( obit_get(*b1) == obit_get(*b2))
			*b3 = obit_set(0);
		else
			*b3 = obit_set(1);
}

void obit_or(unsigned char *b1, unsigned char *b2, unsigned char *b3) {
	if( obit_get(*b1) == 1 || obit_get(*b2) == 1)
		*b3 = obit_set(1);
	else
		*b3 = obit_set(0);

}

void obit_and(unsigned char *b1, unsigned char *b2, unsigned char *b3) {
	if( obit_get(*b1) == 1 && obit_get(*b2) == 1)
		*b3 = obit_set(1);
	else
		*b3 = obit_set(0);

}

/* obit_not(b1, b2)
 * 1. b2 = NOT(b1)
 */
void obit_not(unsigned char *b1, unsigned char *b2) {
	if( obit_get(*b1) == 1)
		*b2 = obit_set(0);
	else
		*b2 = obit_set(1);
}

/* obit_add(b1, b2, b3, carry)
 * b3 = b1 + b2, carry holds bit carry for the sum
 */
void obit_add(unsigned char *b1, unsigned char *b2,
		unsigned char *b3, unsigned char *carry) {
	unsigned char e1, e2, c;
	int i;
	unsigned char add_table[8][5]={
	/*	 e1 e2 c   s  c */
		{0, 0, 0,  0, 0},
		{0, 1, 0,  1, 0},
		{1, 0, 0,  1, 0},
		{1, 1, 0,  0, 1},
		{0, 0, 1,  1, 0},
		{0, 1, 1,  0, 1},
		{1, 0, 1,  0, 1},
		{1, 1, 1,  1, 1} };

	e1 = obit_get(*b1);
	e2 = obit_get(*b2);
	c = obit_get(*carry);

	for(i=0; i<8; i++) {
		unsigned char t1, t2, t3, t4, t5;
		t1 = add_table[i][0];
		t2 = add_table[i][1];
		t3 = add_table[i][2];
		t4 = add_table[i][3];
		t5 = add_table[i][4];
		if( e1 == t1 && e2 == t2 && c == t3) {
			*b3    = obit_set(t4);
			*carry = obit_set(t5);
			return;
		}
	}

	return;
}

/* obyte_xor(ob1, ob2, ob3)
 * 1. ob3 = ob1 XOR ob2
 */
void obyte_xor(struct obyte *ob1, struct obyte *ob2, struct obyte *ob3) {
	int i;
	for(i=0; i<8; i++) {
		obit_xor( &ob1->s[i], &ob2->s[i], &ob3->s[i] );
	}
}


/************************/
/* Arithmetic operators */
/************************/

/* obyte_add(ob1, ob2, ob3)
 * 1. ob3 = (ob1 + ob2) mod 256
 */
void obyte_add(struct obyte *ob1, struct obyte *ob2, struct obyte *ob3) {
	int i;
	unsigned char carry; /* this is carry obit */
	struct obyte tmp;
	struct obyte tmpc;

	carry = obit_set(0);

	__asm__ __volatile__
		("nop
		  nop
		  nop
		  nop
		  nop
		  nop
		  nop
		  nop
		  nop");

	for(i=0; i<8; i++)
		obit_add(&ob1->s[i], &ob2->s[i], &ob3->s[i], &carry);
	while( obit_get(carry) != 0 ) {
		carry = obit_set(0);
		obyte_set(&tmpc, obit_get(carry));
		obyte_add(ob3, &tmpc, &tmp);
		obyte_copy(ob3, &tmp);
	}

	return;

}

/*******************/
/* Other operators */
/*******************/

/* obyte copy(dst, src)
 * 1. dst = src
 */
void obyte_copy(struct obyte *dst, struct obyte *src) {
	int i;
	for(i=0; i<8; i++)
		dst->s[i] = src->s[i];
}

/* obyte_swap(ob1, ob2)
 * 1. tmp = ob1
 * 2. ob1 = ob2
 * 3. ob2 = tmp
 */
void obyte_swap(struct obyte *ob1, struct obyte *ob2) {
	struct obyte tmp;

	obyte_copy( &tmp, ob2);
	obyte_copy( ob2, ob1);
	obyte_copy( ob1, &tmp);
}
