/* OBRC4 1.0 by Pawel Krawczyk <kravietz at echelon dot pl>
 *
 * This is a RC4 implementation for OBCODE. It has many
 * limitations in the sense that its security could be made
 * much better. Currently, it doesn't entirely operate in OBCODE
 * because of lack of some operators, but it should be still
 * more difficult to reverse engineer.
 *
 * Redistribution of this file is permitted under the Lesser GNU Public License.
 *
 * $Id: obrc4.c,v 1.2 2002/12/16 21:00:44 kravietz Exp $
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "obcode.h"

/* #define DEBUG_CRYPT */
/* #define DEBUG_KEY_SCHEDULE */

struct obrc4_key {
	struct obyte state[256];
	struct obyte x;
	struct obyte y;
};

void obrc4_key_schedule(struct obyte *k, unsigned int kl,
		struct obrc4_key *obrc4_key) {
	int index1, index2, i;

	/* initialize state table */
	for(i=0; i<256; i++)
		obyte_set(&obrc4_key->state[i], i);
	obyte_set(&obrc4_key->x, 0);
	obyte_set(&obrc4_key->y, 0);

	index1 = 0;
	index2 = 0;

	/* mix the state table with provided key */
	for(i=0; i<256; i++) {
		unsigned char kc;
		unsigned char rs;

		/* index2 = (key_data_ptr[index1] + state[counter] */
		/*          + index2) % 256; */
		kc = obyte_get(&k[index1]);
		rs = obyte_get(&obrc4_key->state[i]);
		index2 = (kc + rs + index2) % 256;
#ifdef DEBUG_KEY_SCHEDULE
		printf("kc=%02x rs=%02x index2=%02x ", kc, rs, index2);
#endif

		/* swap_byte(&state[counter], &state[index2]); */
		obyte_swap( &obrc4_key->state[i], &obrc4_key->state[index2]);

		/* index1 = (index1 + 1) % key_data_len; */
		index1 = (index1 + 1) % kl;

#ifdef DEBUG_KEY_SCHEDULE
		printf("%02x\n", obyte_get( &obrc4_key->state[i]));
#endif

		
	}
} /* end of key schedule */

void obrc4_crypt(unsigned char *buf, unsigned int len, struct obrc4_key *key) {
	int i;
	unsigned char x, y;

	x = obyte_get(&key->x);
	y = obyte_get(&key->y);
#ifdef DEBUG_CRYPT
	printf("x=%02x y=%02x\n", x, y);
#endif

	for(i=0; i<len; i++) {
		unsigned char xorindex;

		x = (x + 1) % 256;
		y = ( obyte_get(&key->state[x]) + y) % 256;
#ifdef DEBUG_CRYPT
		printf("state[%02x]=%02x\n", x, obyte_get(&key->state[x]) );
#endif
		obyte_swap( &key->state[x], &key->state[y] );
		xorindex = ( obyte_get(&key->state[x])
				+ obyte_get(&key->state[y]) ) % 256;
		buf[i] ^= obyte_get( &key->state[xorindex] );
#ifdef DEBUG_CRYPT
		printf("x=%02x y=%02x xorindex=%02x\n", x, y, xorindex);
#endif


	}

} /* obrc4_crypt */

int main(int argc, char **argv) {
	struct obrc4_key obrc4_key;
	struct obyte *key;
	unsigned char buf[128];
	unsigned char *p;
	int r, i, len;

	if(argc < 2) {
		printf("obrc4 key <in >out\n");
		return 1;
	}

	obcode_init(0);

	p = argv[1];
	len = strlen(p);
	key = malloc(sizeof(struct obyte)*len);
	for(i=0; i<len; i++)
		obyte_set( &key[i], *(p+i));

	obrc4_key_schedule(key, 6, &obrc4_key);

	while( (r = read(0, buf, sizeof(buf))) > 0) {
		obrc4_crypt(buf, r, &obrc4_key);
		write(1, buf, r);
	}

	obcode_finish();
	return 0;
}
