/* OBCODE 1.0 by Pawel Krawczyk <kravietz at echelon dot pl>
 *
 * Random driver for the OBCODE library. The library
 * utilizes random numbers to encode obits. This drivers
 * delivers them.
 *
 * The random numbers need not to be cryptographically
 * strong, so we just use rand() seeded from any best source
 * of entropy we can find in the system.
 *
 * Redistribution of this file is permitted under the Lesser GNU Public License.
 *
 * $Id: random.c,v 1.2 2002/12/16 21:00:44 kravietz Exp $
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

int obcode_sys_random_fd = -1;

void obcode_sys_random_init(void) {
	unsigned int seed;
	int r;

	/* Read seed from urandom if it's available */
	obcode_sys_random_fd = open("/dev/urandom", O_RDONLY);
	if(obcode_sys_random_fd > 0) {
		r = read(obcode_sys_random_fd, &seed, sizeof(seed));
		close(obcode_sys_random_fd);
	}
	/* Modify the seed with time and PID, if it's read
	 * from urandom then it's OK, when it's not then it's
	 * also enough. There's also chance that it was in
	 * uninitialised state before.
	 */
	seed ^= time(NULL)^getpid();
		
	srand(seed);
}

void obcode_sys_random_finish(void) {
	return;
}

unsigned char obcode_sys_random_byte(void) {
	union {
		int i;
		unsigned char c[sizeof(int)];
	} b;

	b.i = rand();


	return b.c[sizeof(int)-1];
}
