#!/bin/sh
# sysdeps.sh by Pawel Krawczyk
#
# This script tries to detect compiler flags best for the host
# system. This is currently gcc and Linux oriented. It tries to
# use CPU instructions best for present processor and moderate
# compiler optimizations. HIGH OPTIMIZATIONS MAY RENDER ANTI-DEBUGGING
# TRICKS INEFFECTIVE!
#
# This script is covered by the GNU Public License (unlike the library code).
#
# $Id: sysdeps.sh,v 1.2 2002/12/16 21:00:44 kravietz Exp $

exec 2>/dev/null

echo 'int main(void) { return 0; }' >sysdeps.c

CFLAGS="-O"

VERSION="`cat VERSION`"
echo "#define VERSION \"$VERSION\"" >version.h

CPU=""
if gcc -m486 -c sysdeps.c; then
	CPU="486"
fi
if gcc -mcpu=i486 -c sysdeps.c; then
	CPU="i486"
fi
if gcc -mcpu=i586 -c sysdeps.c; then
	CPU="i586 $CPU"
fi
if gcc -mcpu=pentium-mmx -c sysdeps.c; then
	CPU="pentium-mmx $CPU"
fi
if gcc -mcpu=i686 -c sysdeps.c; then
	CPU="i686 $CPU"
fi
if gcc -mcpu=k6 -c sysdeps.c; then
	CPU="k6 $CPU"
fi
if gcc -mcpu=athlon -c sysdeps.c; then
	CPU="athlon $CPU"
fi
if gcc -mcpu=pentium2 -c sysdeps.c; then
	CPU="pentium2 $CPU"
fi
if gcc -mcpu=pentium3 -c sysdeps.c; then
	CPU="pentium3 $CPU"
fi

if gcc -fomit-frame-pointer -c sysdeps.c; then
	CFLAGS="-fomit-frame-pointer $CFLAGS"
fi

# This is far from being complete.
sys=`uname -m`
if [ "$sys" = "i586" ]; then
	if egrep -q 'model name.*Pentium' /proc/cpuinfo; then
		if [ "`echo $CPU|grep i586`" != "" ]; then
			CFLAGS="-mcpu=i586 $CFLAGS"
		fi
	fi
	if egrep -q 'flags.*mmx' /proc/cpuinfo; then
		if [ "`echo $CPU|grep pentium-mmx`" != "" ]; then
			CFLAGS="-mcpu=pentium-mmx $CFLAGS"
		fi
	fi
	if egrep -q 'model name.*AMD' /proc/cpuinfo; then
		if [ "`echo $CPU|grep k6`" != "" ]; then
			CFLAGS="-mcpu=k6 $CFLAGS"
		fi
	fi
fi
if [ "$sys" = "i686" ]; then
	if egrep -q 'model name.*AMD' /proc/cpuinfo; then
		if [ "`echo $CPU|grep athlon`" != "" ]; then
			CFLAGS="-mcpu=athlon $CFLAGS"
		fi
	fi
	if egrep -q 'model name.*Pentium II ' /proc/cpuinfo; then
		if [ "`echo $CPU|grep pentium2`" != "" ]; then
			CFLAGS="-mcpu=pentium2 $CFLAGS"
		fi
	fi
	if egrep -q 'model name.*Pentium III ' /proc/cpuinfo; then
		if [ "`echo $CPU|grep pentium3`" != "" ]; then
			CFLAGS="-mcpu=pentium3 $CFLAGS"
		fi
	fi
fi

rm -f sysdeps.c sysdeps.o
echo "CFLAGS+=$CFLAGS"
echo "CC=gcc"
