/* OBTEST 1.0 by Pawel Krawczyk <kravietz at echelon dot pl>
 *
 * Testing procedure for basic operations in OBCODE.
 *
 * Redistribution of this file is permitted under the Lesser GNU Public License.
 * 
 * $Id: test.c,v 1.2 2002/12/16 21:00:44 kravietz Exp $
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "obcode.h"

int main(void) {
	struct obyte obyte;
	struct obyte ob1;
	struct obyte ob2;
	struct obyte ob3;
	unsigned char b,c ;
	int i,j;
	unsigned char rc4_key[]="abcdef";
	unsigned char plaintext[]={ '0', '1', '2', '3', '4', '5', '6', '7' };
	unsigned char correct_ciphertext[]={ 
		0x26, 0x11, 0x33, 0x48, 0x26, 0x0a, 0x7a, 0xa4
	};
	struct obyte ob_rc4_key[sizeof(rc4_key)];
	struct obyte rc4_state[256];
	unsigned char x;
	unsigned char y;
	unsigned char index1;
	unsigned char  index2;

	obcode_init(0);

	b = 0;
	i = 0;
	while(i<10) {
		unsigned char c;
		c = b;
		printf("obit_set(%d)=", c);
		c = obit_set(c);
		printf("0x%02x, obit_get(0x%02x)=", c, c);
		c = obit_get(c);
		printf("%d\n", c);
		if(c != b) {
			printf("ERROR!\n");
			return 1;
		}
		i++;
		b = (b ? 0 : 1);
	}

	b = 0;
	printf("obyte_set(0x%02x)=", b);
	obyte_set(&obyte, b);
	for(i=0; i<8; i++) printf("%02x", obyte.s[i]);
	b = obyte_get(&obyte);
	printf(" obyte_get()=%02x\n", b);
	if(b != 0) { printf("ERROR!\n"); exit(1); }

	b = 1;
	printf("obyte_set(0x%02x)=", b);
	obyte_set(&obyte, b);
	for(i=0; i<8; i++) printf("%02x", obyte.s[i]);
	b = obyte_get(&obyte);
	printf(" obyte_get()=%02x\n", b);
	if(b != 1) { printf("ERROR!\n"); exit(1); }

	b = 0xff;
	printf("obyte_set(0x%02x)=", b);
	obyte_set(&obyte, b);
	for(i=0; i<8; i++) printf("%02x", obyte.s[i]);
	b = obyte_get(&obyte);
	printf(" obyte_get()=%02x\n", b);
	if(b != 0xff) { printf("ERROR!\n"); exit(1); }

	for(i=0; i<10; i++) {
		int j;
		unsigned char stor;

		stor = b = obit_set(1);	/* Obtain a random byte */
		printf("obyte_set(0x%02x)=", b);
		obyte_set(&obyte, b);
		for(j=0; j<8; j++) printf("%02x", obyte.s[j]);
		b = obyte_get(&obyte);
		printf(" obyte_get()=%02x\n", b);
		if(b != stor) { printf("ERROR!\n"); exit(1); }
	}

	b = 0xa5^0xff;
	obyte_set(&ob1, 0xa5); /* 10100101 */
	printf("xor(0xa5, 0xff)=0x%02x\n", b);
	printf(" obyte_xor(");
	for(j=0; j<8; j++) printf("%02x", ob1.s[j]);
	obyte_set(&ob2, 0xff);
	printf(",");
	for(j=0; j<8; j++) printf("%02x", ob2.s[j]);
	printf(")=");
	obyte_xor(&ob1, &ob2, &ob3);
	for(j=0; j<8; j++) printf("%02x", ob3.s[j]);
	c = obyte_get(&ob3);
	printf("\n obyte_get()=0x%02x\n", c);
	if(c != b ) { printf("ERROR!\n"); exit(1); }

	for(i=1; i<10; i++) {
		unsigned char num;
		unsigned char add;
		unsigned char res;

		num = obit_set(1);	/* random */
		add = obit_set(1);	/* random */
		res = num + add;
		printf("add(%u, %u)=%u\n", num, add, res);
		printf(" obyte_add(");
		obyte_set(&ob1, num);
		for(j=0; j<8; j++) printf("%02x", ob1.s[j]);
		printf(",%d", add);
		obyte_set(&ob2, add);
		obyte_add(&ob1, &ob2, &ob3);
		printf(")=");
		for(j=0; j<8; j++) printf("%02x", ob3.s[j]);
		printf("\n obyte_get()=%d\n", obyte_get(&ob3));
		if(obyte_get(&ob3) != res) {
			printf("ERROR!\n"); exit(1);
		}
	}

	/* RC4 */
	/* prepare key */
	for(i=0; i<sizeof(rc4_key); i++) /* convert byte key to obyte key */
		obyte_set( &ob_rc4_key[i], rc4_key[i]);
	for(i=0; i<256; i++) /* initialize state table */
		obyte_set( &rc4_state[i], i);
	index1 = 0;
	index2 = 0;
	for(i=0; i<256; i++) {

		/* index2 = (key_data_ptr[index1] + state[counter] + index2) % 256; */
		index2 = (rc4_key[index1] + obyte_get(&rc4_state[i]) + index2) % 256;
#if 0
		printf("index2=%02x ", index2);
#endif

		/* swap_byte(&state[counter], &state[index2]); */
		obyte_swap( &rc4_state[i], &rc4_state[index2]);

		/* index1 = (index1 + 1) % key_data_len; */
		index1 = (index1+1) % sizeof(rc4_key);
#if 0		
		printf("%02x\n", obyte_get( &rc4_state[i]));
#endif
		
	}
	printf("\n");

	/* encrypt */
	x = y = 0;
	for(i=0; i<sizeof(plaintext); i++) {
		unsigned char xorindex;
		x = (x+1) % 256;
		y = ( obyte_get(&rc4_state[x]) + y) % 256;
		obyte_copy( &ob2, &rc4_state[x]);
		obyte_copy( &ob3, &rc4_state[y]);
		obyte_copy( &rc4_state[y], &ob2);
		obyte_copy( &rc4_state[x], &ob3);
		xorindex = ( obyte_get(&rc4_state[x])+obyte_get(&rc4_state[y])) % 256;
		plaintext[i] ^= obyte_get( &rc4_state[xorindex] );


	}

	printf("\nRC4=");
	for(j=0; j<8; j++) printf("%02x", plaintext[j]);
	printf("\n");
	if( memcmp(plaintext, correct_ciphertext, sizeof(plaintext)) != 0) {
		printf("ERROR!\n");
		exit(1);
	}



	obcode_finish();

	return 0;
}
